/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Much of this is taken from binutils and GNU libc ...
 * Copyright (C) 2020 Loongson Technology Co., Ltd.
 */
#ifndef _ASM_ELF_H
#define _ASM_ELF_H

#include <linux/auxvec.h>
#include <linux/fs.h>
#include <linux/mm_types.h>

#include <uapi/linux/elf.h>

#include <asm/current.h>

/* ELF header e_flags defines. */

/* The ABI of a file. */
#define EF_LOONGARCH_ABI_LP32		0x00000001	/* LP32 ABI.  */
#define EF_LOONGARCH_ABI_LPX32		0x00000002	/* LPX32 ABI  */
#define EF_LOONGARCH_ABI_LP64		0x00000003	/* LP64 ABI  */
#define EF_LOONGARCH_ABI		0x00000003

#define PT_LOONGARCH_REGINFO		0x70000000
#define PT_LOONGARCH_RTPROC		0x70000001
#define PT_LOONGARCH_OPTIONS		0x70000002
#define PT_LOONGARCH_ABIFLAGS	0x70000003

#define DT_LOONGARCH_RLD_VERSION	0x70000001
#define DT_LOONGARCH_TIME_STAMP	0x70000002
#define DT_LOONGARCH_ICHECKSUM	0x70000003
#define DT_LOONGARCH_IVERSION	0x70000004
#define DT_LOONGARCH_FLAGS		0x70000005
	#define RHF_NONE	0x00000000
	#define RHF_HARDWAY	0x00000001
	#define RHF_NOTPOT	0x00000002
	#define RHF_SGI_ONLY	0x00000010
#define DT_LOONGARCH_BASE_ADDRESS	0x70000006
#define DT_LOONGARCH_CONFLICT	0x70000008
#define DT_LOONGARCH_LIBLIST		0x70000009
#define DT_LOONGARCH_LOCAL_GOTNO	0x7000000a
#define DT_LOONGARCH_CONFLICTNO	0x7000000b
#define DT_LOONGARCH_LIBLISTNO	0x70000010
#define DT_LOONGARCH_SYMTABNO	0x70000011
#define DT_LOONGARCH_UNREFEXTNO	0x70000012
#define DT_LOONGARCH_GOTSYM		0x70000013
#define DT_LOONGARCH_HIPAGENO	0x70000014
#define DT_LOONGARCH_RLD_MAP		0x70000016

#define R_LOONGARCH_NONE		0
#define R_LOONGARCH_16		1
#define R_LOONGARCH_32		2
#define R_LOONGARCH_REL32		3
#define R_LOONGARCH_26		4
#define R_LOONGARCH_HI16		5
#define R_LOONGARCH_LO16		6
#define R_LOONGARCH_GPREL16		7
#define R_LOONGARCH_LITERAL		8
#define R_LOONGARCH_GOT16		9
#define R_LOONGARCH_PC16		10
#define R_LOONGARCH_CALL16		11
#define R_LOONGARCH_GPREL32		12
/* The remaining relocs are defined on Irix, although they are not
   in the LOONGARCH ELF ABI.	 */
#define R_LOONGARCH_UNUSED1		13
#define R_LOONGARCH_UNUSED2		14
#define R_LOONGARCH_UNUSED3		15
#define R_LOONGARCH_SHIFT5		16
#define R_LOONGARCH_SHIFT6		17
#define R_LOONGARCH_64		18
#define R_LOONGARCH_GOT_DISP		19
#define R_LOONGARCH_GOT_PAGE		20
#define R_LOONGARCH_GOT_OFST		21
/*
 * The following two relocation types are specified in the LOONGARCH ABI
 * conformance guide version 1.2 but not yet in the psABI.
 */
#define R_LOONGARCH_GOTHI16		22
#define R_LOONGARCH_GOTLO16		23
#define R_LOONGARCH_SUB		24
#define R_LOONGARCH_INSERT_A		25
#define R_LOONGARCH_INSERT_B		26
#define R_LOONGARCH_DELETE		27
#define R_LOONGARCH_HIGHER		28
#define R_LOONGARCH_HIGHEST		29
/*
 * The following two relocation types are specified in the LOONGARCH ABI
 * conformance guide version 1.2 but not yet in the psABI.
 */
#define R_LOONGARCH_CALLHI16		30
#define R_LOONGARCH_CALLLO16		31
/*
 * Introduced for LOONGARCHr6.
 */
#define R_LOONGARCH_PC21_S2		60
#define R_LOONGARCH_PC26_S2		61
/*
 * This range is reserved for vendor specific relocations.
 */
#define R_LOONGARCH_LOVENDOR		100
#define R_LOONGARCH_HIVENDOR		127

/* LA relocation types used by the dynamic linker */
#define R_LA_NONE			0
#define R_LA_32				1
#define R_LA_64				2
#define R_LA_MARK_LA			20
#define R_LA_MARK_PCREL			21
#define R_LA_SOP_PUSH_PCREL		22
#define R_LA_SOP_PUSH_ABSOLUTE		23
#define R_LA_SOP_PUSH_PLT_PCREL		29
#define R_LA_SOP_SUB			32
#define R_LA_SOP_SL			33
#define R_LA_SOP_SR			34
#define R_LA_SOP_ADD			35
#define R_LA_SOP_AND			36
#define R_LA_SOP_IF_ELSE		37
#define R_LA_SOP_POP_32_S_10_5		38
#define R_LA_SOP_POP_32_U_10_12		39
#define R_LA_SOP_POP_32_S_10_12		40
#define R_LA_SOP_POP_32_S_10_16		41
#define R_LA_SOP_POP_32_S_10_16_S2	42
#define R_LA_SOP_POP_32_S_5_20		43
#define R_LA_SOP_POP_32_S_0_5_10_16_S2	44
#define R_LA_SOP_POP_32_S_0_10_10_16_S2	45
#define R_LA_SOP_POP_32_U		46
#define R_LA_ADD64			51
#define R_LA_SUB64			56

#define SHN_LOONGARCH_ACCOMON	0xff00		/* Allocated common symbols */
#define SHN_LOONGARCH_TEXT		0xff01		/* Allocated test symbols.  */
#define SHN_LOONGARCH_DATA		0xff02		/* Allocated data symbols.  */
#define SHN_LOONGARCH_SCOMMON	0xff03		/* Small common symbols */
#define SHN_LOONGARCH_SUNDEFINED	0xff04		/* Small undefined symbols */

#define SHT_LOONGARCH_LIST		0x70000000
#define SHT_LOONGARCH_CONFLICT	0x70000002
#define SHT_LOONGARCH_GPTAB		0x70000003
#define SHT_LOONGARCH_UCODE		0x70000004
#define SHT_LOONGARCH_DEBUG		0x70000005
#define SHT_LOONGARCH_REGINFO	0x70000006
#define SHT_LOONGARCH_PACKAGE	0x70000007
#define SHT_LOONGARCH_PACKSYM	0x70000008
#define SHT_LOONGARCH_RELD		0x70000009
#define SHT_LOONGARCH_IFACE		0x7000000b
#define SHT_LOONGARCH_CONTENT	0x7000000c
#define SHT_LOONGARCH_OPTIONS	0x7000000d
#define SHT_LOONGARCH_SHDR		0x70000010
#define SHT_LOONGARCH_FDESC		0x70000011
#define SHT_LOONGARCH_EXTSYM		0x70000012
#define SHT_LOONGARCH_DENSE		0x70000013
#define SHT_LOONGARCH_PDESC		0x70000014
#define SHT_LOONGARCH_LOCSYM		0x70000015
#define SHT_LOONGARCH_AUXSYM		0x70000016
#define SHT_LOONGARCH_OPTSYM		0x70000017
#define SHT_LOONGARCH_LOCSTR		0x70000018
#define SHT_LOONGARCH_LINE		0x70000019
#define SHT_LOONGARCH_RFDESC		0x7000001a
#define SHT_LOONGARCH_DELTASYM	0x7000001b
#define SHT_LOONGARCH_DELTAINST	0x7000001c
#define SHT_LOONGARCH_DELTACLASS	0x7000001d
#define SHT_LOONGARCH_DWARF		0x7000001e
#define SHT_LOONGARCH_DELTADECL	0x7000001f
#define SHT_LOONGARCH_SYMBOL_LIB	0x70000020
#define SHT_LOONGARCH_EVENTS		0x70000021
#define SHT_LOONGARCH_TRANSLATE	0x70000022
#define SHT_LOONGARCH_PIXIE		0x70000023
#define SHT_LOONGARCH_XLATE		0x70000024
#define SHT_LOONGARCH_XLATE_DEBUG	0x70000025
#define SHT_LOONGARCH_WHIRL		0x70000026
#define SHT_LOONGARCH_EH_REGION	0x70000027
#define SHT_LOONGARCH_XLATE_OLD	0x70000028
#define SHT_LOONGARCH_PDR_EXCEPTION	0x70000029

#define SHF_LOONGARCH_GPREL		0x10000000
#define SHF_LOONGARCH_MERGE		0x20000000
#define SHF_LOONGARCH_ADDR		0x40000000
#define SHF_LOONGARCH_STRING		0x80000000
#define SHF_LOONGARCH_NOSTRIP	0x08000000
#define SHF_LOONGARCH_LOCAL		0x04000000
#define SHF_LOONGARCH_NAMES		0x02000000
#define SHF_LOONGARCH_NODUPES	0x01000000

#define LOONGARCH_ABI_FP_ANY		0	/* FP ABI doesn't matter */
#define LOONGARCH_ABI_FP_DOUBLE	1	/* -mdouble-float */
#define LOONGARCH_ABI_FP_SINGLE	2	/* -msingle-float */
#define LOONGARCH_ABI_FP_SOFT	3	/* -msoft-float */

struct loongarch_elf_abiflags_v0 {
	uint16_t version;	/* Version of flags structure */
	uint8_t isa_level;	/* The level of the ISA: 1-5, 32, 64 */
	uint8_t isa_rev;	/* The revision of ISA: 0 for LOONGARCH V and below,
				   1-n otherwise */
	uint8_t gpr_size;	/* The size of general purpose registers */
	uint8_t cpr1_size;	/* The size of co-processor 1 registers */
	uint8_t cpr2_size;	/* The size of co-processor 2 registers */
	uint8_t fp_abi;		/* The floating-point ABI */
	uint32_t isa_ext;	/* Mask of processor-specific extensions */
	uint32_t ases;		/* Mask of ASEs used */
	uint32_t flags1;	/* Mask of general flags */
	uint32_t flags2;
};

#ifndef ELF_ARCH
/* ELF register definitions */
#define ELF_NGREG	45
#define ELF_NFPREG	33

typedef unsigned long elf_greg_t;
typedef elf_greg_t elf_gregset_t[ELF_NGREG];

typedef double elf_fpreg_t;
typedef elf_fpreg_t elf_fpregset_t[ELF_NFPREG];

void loongarch_dump_regs64(u64 *uregs, const struct pt_regs *regs);

#ifdef CONFIG_32BIT
/*
 * This is used to ensure we don't load something for the wrong architecture.
 */
#define elf_check_arch elf32_check_arch

/*
 * These are used to set parameters in the core dumps.
 */
#define ELF_CLASS	ELFCLASS32

#define ELF_CORE_COPY_REGS(dest, regs) \
	loongarch_dump_regs32((u32 *)&(dest), (regs));

#endif /* CONFIG_32BIT */

#ifdef CONFIG_64BIT
/*
 * This is used to ensure we don't load something for the wrong architecture.
 */
#define elf_check_arch elf64_check_arch

/*
 * These are used to set parameters in the core dumps.
 */
#define ELF_CLASS	ELFCLASS64

#define ELF_CORE_COPY_REGS(dest, regs) \
	loongarch_dump_regs64((u64 *)&(dest), (regs));

#endif /* CONFIG_64BIT */

/*
 * These are used to set parameters in the core dumps.
 */
#define ELF_DATA	ELFDATA2LSB
#define ELF_ARCH	EM_LOONGARCH

#endif /* !defined(ELF_ARCH) */

#define loongarch_elf_check_machine(x) ((x)->e_machine == EM_LOONGARCH)

#define vmcore_elf32_check_arch loongarch_elf_check_machine
#define vmcore_elf64_check_arch loongarch_elf_check_machine

/*
 * Return non-zero if HDR identifies an 32bit ELF binary.
 */
#define elf32_check_arch(hdr)						\
({									\
	int __res = 1;							\
	struct elfhdr *__h = (hdr);					\
									\
	if (!loongarch_elf_check_machine(__h))				\
		__res = 0;						\
	if (__h->e_ident[EI_CLASS] != ELFCLASS32)			\
		__res = 0;						\
									\
	__res;								\
})

/*
 * Return non-zero if HDR identifies an 64bit ELF binary.
 */
#define elf64_check_arch(hdr)						\
({									\
	int __res = 1;							\
	struct elfhdr *__h = (hdr);					\
									\
	if (!loongarch_elf_check_machine(__h))				\
		__res = 0;						\
	if (__h->e_ident[EI_CLASS] != ELFCLASS64)			\
		__res = 0;						\
									\
	__res;								\
})

struct loongarch_abi;
extern struct loongarch_abi loongarch_abi;

#ifdef CONFIG_32BIT

#define SET_PERSONALITY2(ex, state)					\
do {									\
	current->thread.abi = &loongarch_abi;				\
									\
	loongarch_set_personality_nan(state);				\
									\
	if (personality(current->personality) != PER_LINUX)		\
		set_personality(PER_LINUX);				\
} while (0)

#endif /* CONFIG_32BIT */

#ifdef CONFIG_64BIT

#define SET_PERSONALITY2(ex, state)					\
do {									\
	unsigned int p;							\
									\
	clear_thread_flag(TIF_32BIT_REGS);				\
	clear_thread_flag(TIF_32BIT_ADDR);				\
									\
	current->thread.abi = &loongarch_abi;			\
	loongarch_set_personality_nan(state);				\
									\
	p = personality(current->personality);				\
	if (p != PER_LINUX32 && p != PER_LINUX)				\
		set_personality(PER_LINUX);				\
} while (0)

#endif /* CONFIG_64BIT */

#define CORE_DUMP_USE_REGSET
#define ELF_EXEC_PAGESIZE	PAGE_SIZE

/* This yields a mask that user programs can use to figure out what
   instruction set this cpu supports.  This could be done in userspace,
   but it's not easy, and we've already done it here.  */

#define ELF_HWCAP	(elf_hwcap)
extern unsigned int elf_hwcap;
#include <asm/hwcap.h>

/*
 * This yields a string that ld.so will use to load implementation
 * specific libraries for optimization.	 This is more specific in
 * intent than poking at uname or /proc/cpuinfo.
 */

#define ELF_PLATFORM  __elf_platform
extern const char *__elf_platform;

/*
 * See comments in asm-alpha/elf.h, this is the same thing
 * on the LOONGARCH.
 */
#define ELF_PLAT_INIT(_r, load_addr)	do { \
	_r->regs[1] = _r->regs[2] = _r->regs[3] = _r->regs[4] = 0;	\
	_r->regs[5] = _r->regs[6] = _r->regs[7] = _r->regs[8] = 0;	\
	_r->regs[9] = _r->regs[10] = _r->regs[11] = _r->regs[12] = 0;	\
	_r->regs[13] = _r->regs[14] = _r->regs[15] = _r->regs[16] = 0;	\
	_r->regs[17] = _r->regs[18] = _r->regs[19] = _r->regs[20] = 0;	\
	_r->regs[21] = _r->regs[22] = _r->regs[23] = _r->regs[24] = 0;	\
	_r->regs[25] = _r->regs[26] = _r->regs[27] = _r->regs[28] = 0;	\
	_r->regs[30] = _r->regs[31] = 0;				\
} while (0)

/* This is the location that an ET_DYN program is loaded if exec'ed.  Typical
   use of this is to invoke "./ld.so someprog" to test out a new version of
   the loader.	We need to make sure that it is out of the way of the program
   that it will "exec", and that there is sufficient room for the brk.	*/

#ifndef ELF_ET_DYN_BASE
#define ELF_ET_DYN_BASE		(TASK_SIZE / 3 * 2)
#endif

/* update AT_VECTOR_SIZE_ARCH if the number of NEW_AUX_ENT entries changes */
#define ARCH_DLINFO							\
do {									\
	NEW_AUX_ENT(AT_SYSINFO_EHDR,					\
		    (unsigned long)current->mm->context.vdso);		\
} while (0)

#define ARCH_HAS_SETUP_ADDITIONAL_PAGES 1
struct linux_binprm;
extern int arch_setup_additional_pages(struct linux_binprm *bprm,
				       int uses_interp);

struct arch_elf_state {
	int nan_2008;
	int fp_abi;
	int interp_fp_abi;
	int overall_fp_mode;
};

#define LOONGARCH_ABI_FP_UNKNOWN	(-1)	/* Unknown FP ABI (kernel internal) */

#define INIT_ARCH_ELF_STATE {			\
	.nan_2008 = -1,				\
	.fp_abi = LOONGARCH_ABI_FP_UNKNOWN,		\
	.interp_fp_abi = LOONGARCH_ABI_FP_UNKNOWN,	\
	.overall_fp_mode = -1,			\
}

/* Whether to accept legacy-NaN and 2008-NaN user binaries.  */
extern bool loongarch_use_nan_legacy;
extern bool loongarch_use_nan_2008;

extern int arch_elf_pt_proc(void *ehdr, void *phdr, struct file *elf,
			    bool is_interp, struct arch_elf_state *state);

extern int arch_check_elf(void *ehdr, bool has_interpreter, void *interp_ehdr,
			  struct arch_elf_state *state);

extern void loongarch_set_personality_nan(struct arch_elf_state *state);

#define elf_read_implies_exec(ex, stk) loongarch_elf_read_implies_exec(&(ex), stk)
extern int loongarch_elf_read_implies_exec(void *elf_ex, int exstack);

#endif /* _ASM_ELF_H */
