/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Copyright (C) 2013, Loongson Technology Corporation Limited, Inc.
 *  Copyright (C) 2014-2017, Lemote, Inc.
 */
#ifndef _LOONGSON_PCH_H
#define _LOONGSON_PCH_H

#include <linux/msi.h>
#include <linux/pci.h>
#include <linux/types.h>
#include <asm/addrspace.h>

/* ============== LS7A registers =============== */
#define LS7A_PCH_REG_BASE		0x10000000UL
/* CHIPCFG regs */
#define LS7A_CHIPCFG_REG_BASE		(LS7A_PCH_REG_BASE + 0x00010000)
/* MISC reg base */
#define LS7A_MISC_REG_BASE		(LS7A_PCH_REG_BASE + 0x00080000)
/* ACPI regs */
#define LS7A_ACPI_REG_BASE		(LS7A_MISC_REG_BASE + 0x00050000)
/* RTC regs */
#define LS7A_RTC_REG_BASE		(LS7A_MISC_REG_BASE + 0x00050100)
#define LS7A_PCI_DC_REG_BASE		(LS7A_PCH_REG_BASE + 0x0a000000)

#define LS7A_DMA_CFG	(void *)TO_UNCAC(LS7A_CHIPCFG_REG_BASE + 0x041c)
#define LS7A_DMA_NODE_POS	36
#define LS7A_DMA_NODE_OFF	8
#define LS7A_DMA_NODE_MASK	0x1F00

/* RTC ram addr */
#define LS7A_RTC_RAM	(LS7A_ACPI_REG_BASE + 0x00000050)

#define LS7A_INT_MASK_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x020)
#define LS7A_INT_EDGE_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x060)
#define LS7A_INT_CLEAR_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x080)
#define LS7A_INT_HTMSI_EN_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x040)
#define LS7A_INT_ROUTE_ENTRY_REG	(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x100)
#define LS7A_INT_HTMSI_VEC_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x200)
#define LS7A_INT_STATUS_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x3a0)
#define LS7A_INT_POL_REG		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x3e0)
#define LS7A_LPC_INT_CTL		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x2000)
#define LS7A_LPC_INT_ENA		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x2004)
#define LS7A_LPC_INT_STS		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x2008)
#define LS7A_LPC_INT_CLR		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x200c)
#define LS7A_LPC_INT_POL		(void *)TO_UNCAC(LS7A_PCH_REG_BASE + 0x2010)

/* ============== LS7A VZ registers =============== */

#define LS7A_VZ_PCH_REG_BASE		0xe0010000000UL
/* CHIPCFG regs */
#define LS7A_VZ_CHIPCFG_REG_BASE	(LS7A_VZ_PCH_REG_BASE + 0x00010000)
/* MISC reg base */
#define LS7A_VZ_MISC_REG_BASE		(LS7A_VZ_PCH_REG_BASE + 0x00080000)
/* ACPI regs */
#define LS7A_VZ_ACPI_REG_BASE		(LS7A_VZ_MISC_REG_BASE + 0x00050000)
/* RTC regs */
#define LS7A_VZ_RTC_REG_BASE		(LS7A_VZ_MISC_REG_BASE + 0x00050100)
#define LS7A_VZ_PCI_DC_REG_BASE		(LS7A_VZ_PCH_REG_BASE + 0x0a000000)

/* RTC ram addr */
#define LS7A_VZ_RTC_RAM	(LS7A_VZ_ACPI_REG_BASE + 0x00000050)

#define LS7A_VZ_INT_MASK_REG		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x020)
#define LS7A_VZ_INT_EDGE_REG		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x060)
#define LS7A_VZ_INT_CLEAR_REG		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x080)
#define LS7A_VZ_INT_HTMSI_EN_REG	(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x040)
#define LS7A_VZ_INT_ROUTE_ENTRY_REG	(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x100)
#define LS7A_VZ_INT_HTMSI_VEC_REG	(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x200)
#define LS7A_VZ_INT_STATUS_REG		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x3a0)
#define LS7A_VZ_INT_POL_REG		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x3e0)
#define LS7A_VZ_LPC_INT_CTL		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x2000)
#define LS7A_VZ_LPC_INT_ENA		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x2004)
#define LS7A_VZ_LPC_INT_STS		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x2008)
#define LS7A_VZ_LPC_INT_CLR		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x200c)
#define LS7A_VZ_LPC_INT_POL		(void *)TO_UNCAC(LS7A_VZ_PCH_REG_BASE + 0x2010)


#define LS7A_PMCON_SOC_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x000)
#define LS7A_PMCON_RESUME_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x004)
#define LS7A_PMCON_RTC_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x008)
#define LS7A_PM1_EVT_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x00c)
#define LS7A_PM1_ENA_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x010)
#define LS7A_PM1_CNT_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x014)
#define LS7A_PM1_TMR_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x018)
#define LS7A_P_CNT_REG			(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x01c)
#define LS7A_GPE0_STS_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x028)
#define LS7A_GPE0_ENA_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x02c)
#define LS7A_RST_CNT_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x030)
#define LS7A_WD_SET_REG			(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x034)
#define LS7A_WD_TIMER_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x038)
#define LS7A_THSENS_CNT_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x04c)
#define LS7A_GEN_RTC_1_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x050)
#define LS7A_GEN_RTC_2_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x054)
#define LS7A_DPM_CFG_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x400)
#define LS7A_DPM_STS_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x404)
#define LS7A_DPM_CNT_REG		(void *)TO_UNCAC(LS7A_ACPI_REG_BASE + 0x408)

#define LS7A_PCIE_BAR_BASE(bus, dev, func) \
        readl((void *)TO_UNCAC(LS7A_PCI_DC_REG_BASE | (bus << 16) | (dev << 11) | (func << 8) | 0x10))

/* 7A bridge has a gpio controller in DC space */
#define LS7A_DC_CNT_REG_BASE    (LS7A_PCIE_BAR_BASE(0x0, 0x6, 0x1) & 0xfffffff0)

/*PCI Configration Space Base*/
#define HT1LO_PCICFG_BASE	0x1a000000
#define HT1LO_PCICFG_BASE_TP1	0x1b000000

#define HT1LO_EXT_PCICFG_BASE_FIX	0xefe00000000UL
#define HT1LO_EXT_PCICFG_BASE		\
	(((struct pci_controller *)(bus)->sysdata)->mcfg_addr)
#define HT1LO_EXT_PCICFG_TP1_OFFSET	0x10000000
#define HT1LO_EXT_PCICFG_BASE_TP1	\
	(HT1LO_EXT_PCICFG_BASE + HT1LO_EXT_PCICFG_TP1_OFFSET)


/* REG ACCESS*/
#define ls7a_readb(addr)			  (*(volatile unsigned char  *)TO_UNCAC(addr))
#define ls7a_readw(addr)			  (*(volatile unsigned short *)TO_UNCAC(addr))
#define ls7a_readl(addr)			  (*(volatile unsigned int   *)TO_UNCAC(addr))
#define ls7a_readq(addr)			  (*(volatile unsigned long  *)TO_UNCAC(addr))
#define ls7a_writeb(val, addr)		*(volatile unsigned char  *)TO_UNCAC(addr) = (val)
#define ls7a_writew(val, addr)		*(volatile unsigned short *)TO_UNCAC(addr) = (val)
#define ls7a_writel(val, addr)		ls7a_write_type(val, addr, uint32_t)
#define ls7a_writeq(val, addr)		ls7a_write_type(val, addr, uint64_t)
#define ls7a_write(val, addr)		ls7a_write_type(val, addr, uint64_t)

extern unsigned long ls7a_dc_writeflags;
extern spinlock_t ls7a_dc_writelock;
extern unsigned long ls7a_rwflags;
extern rwlock_t ls7a_rwlock;

#define ls7a_dc_write(val, addr)          \
    do {                                \
        spin_lock_irqsave(&ls7a_dc_writelock,ls7a_dc_writeflags);          \
        *(volatile unsigned long __force *)TO_UNCAC(addr) = (val);          \
        spin_unlock_irqrestore(&ls7a_dc_writelock,ls7a_dc_writeflags);        \
    }while(0)

#define ls7a_read(val, addr)        					  \
    do {                                				  \
        read_lock_irqsave(&ls7a_rwlock,flags); 			          \
        val = *(volatile unsigned long __force *)TO_UNCAC(addr);          \
        read_unlock_irqrestore(&ls7a_rwlock,flags); 		          \
    }while(0)

#define ls7a_write_type(val, addr, type)          					  \
    do {                                				  \
        write_lock_irqsave(&ls7a_rwlock,ls7a_rwflags);          	  \
        *(volatile type __force *)TO_UNCAC(addr) = (val);        \
        write_unlock_irqrestore(&ls7a_rwlock,ls7a_rwflags);               \
    }while(0)

/* ============== Data structrues =============== */

/* gpio data */
struct platform_gpio_data {
        u32 gpio_conf;
        u32 gpio_out;
        u32 gpio_in;
	u32 in_start_bit;
	u32 support_irq;
	char *label;
        int gpio_base;
        int ngpio;
};

extern u32 node_id_offset;

extern struct pci_ops ls7a_pci_ops;
#endif
