/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2020 Loongson Technology Corporation Limited
 */
#ifndef _ASM_PGTABLE_BITS_H
#define _ASM_PGTABLE_BITS_H

/* Page table bits */
enum pgtable_bits {
	_PAGE_VALID_SHIFT,
	_PAGE_DIRTY_SHIFT,
	_PAGE_PLV_SHIFT,
	_CACHE_SHIFT = _PAGE_PLV_SHIFT + 2,
	_PAGE_GLOBAL_SHIFT = _CACHE_SHIFT + 2,
	_PAGE_PSE_SHIFT = _PAGE_GLOBAL_SHIFT,
	_PAGE_PRESENT_SHIFT,
	_PAGE_WRITE_SHIFT,
	_PAGE_HUGE_SHIFT,
	_PAGE_PFN_SHIFT = 12,
	_PAGE_PSE_GLOBAL_SHIFT = 12,
	_PAGE_PFN_END_SHIFT = 48,
	_PAGE_PROTNONE_SHIFT = 59,
	_PAGE_SPECIAL_SHIFT = 60,
	_PAGE_NO_READ_SHIFT = 61,
	_PAGE_NO_EXEC_SHIFT = 62,
	_PAGE_RPLV_SHIFT = 63,
};

/* Used only by software */
#define _PAGE_PRESENT		(1 << _PAGE_PRESENT_SHIFT)
#define _PAGE_WRITE		(1 << _PAGE_WRITE_SHIFT)
#define _PAGE_HUGE		(1 << _PAGE_HUGE_SHIFT)

#define _PAGE_PROTNONE		(_ULCAST_(1)<<_PAGE_PROTNONE_SHIFT)
#define _PAGE_SPECIAL		(_ULCAST_(1)<<_PAGE_SPECIAL_SHIFT)

/* Used by TLB hardware (placed in EntryLo*) */
#define _PAGE_NO_EXEC		(_ULCAST_(1) << _PAGE_NO_EXEC_SHIFT)
#define _PAGE_RPLV			(_ULCAST_(1) << _PAGE_RPLV_SHIFT)
#define _PAGE_NO_READ		(_ULCAST_(1) << _PAGE_NO_READ_SHIFT)
#define _PAGE_GLOBAL		(1 << _PAGE_GLOBAL_SHIFT)
#define _PAGE_VALID		(1 << _PAGE_VALID_SHIFT)
#define _PAGE_DIRTY		(1 << _PAGE_DIRTY_SHIFT)
#define _CACHE_MASK		(3 << _CACHE_SHIFT)
#define _PAGE_PSE_GLOBAL	(1 << _PAGE_PSE_GLOBAL_SHIFT)
#define _PAGE_PSE	(1 << _PAGE_PSE_SHIFT)
#define _PAGE_PLV	(_ULCAST_(3) << _PAGE_PLV_SHIFT)
#define _PFN_SHIFT		(PAGE_SHIFT - 12 + _PAGE_PFN_SHIFT)

#define _PAGE_USER	(PLV_USER << _PAGE_PLV_SHIFT)
#define _PAGE_KERN	(PLV_KERN << _PAGE_PLV_SHIFT)

#define _PAGE_SILENT_READ	_PAGE_VALID
#define _PAGE_SILENT_WRITE	_PAGE_DIRTY

#define _PFN_MASK  (((_ULCAST_(1) << (_PAGE_PFN_END_SHIFT)) - 1) & \
       ~((1 << (_PFN_SHIFT)) - 1))

/*
 * pte_to_entrylo converts a page table entry (PTE) into a Mips
 * entrylo0/1 value.
 */
static inline uint64_t pte_to_entrylo(unsigned long pte_val)
{
	return pte_val;
}

/*
 * Cache attributes
 */

#ifndef _CACHE_SUC
#define _CACHE_SUC			(0<<_CACHE_SHIFT) /* Strong-ordered UnCached */
#endif
#ifndef _CACHE_CC
#define _CACHE_CC			(1<<_CACHE_SHIFT) /* Coherent Cached */
#endif
#ifndef _CACHE_WUC
#define _CACHE_WUC			(2<<_CACHE_SHIFT) /* Weak-ordered UnCached */
#endif

#define __READABLE	(_PAGE_SILENT_READ)
#define __WRITEABLE	(_PAGE_SILENT_WRITE | _PAGE_WRITE)

#define _PAGE_CHG_MASK	(_PAGE_SPECIAL | \
			 _PFN_MASK | _CACHE_MASK | _PAGE_PLV)

#endif /* _ASM_PGTABLE_BITS_H */
